@@ -371,6 +371,7 @@ static void create_aux_windows(_THIS)
 	    classhints->res_name = classname;
 	    classhints->res_class = classname;
 	    XSetClassHint(SDL_Display, WMwindow, classhints);
+            XSetClassHint(SDL_Display, FSwindow, classhints);
 	    XFree(classhints);
 	}
     }
@@ -490,12 +491,12 @@ static void X11_DestroyWindow(_THIS, SDL_Surface *screen)
 
 	if ( ! SDL_windowid ) {
 		/* Hide the managed window */
-		if ( WMwindow ) {
+		if ( !( screen && (screen->flags & SDL_FULLSCREEN) ) && WMwindow ) {
 			XUnmapWindow(SDL_Display, WMwindow);
 		}
 		if ( screen && (screen->flags & SDL_FULLSCREEN) ) {
 			screen->flags &= ~SDL_FULLSCREEN;
-			X11_LeaveFullScreen(this);
+			X11_LeaveFullScreen(this, 1);
 		}
 
 		/* Destroy the output window */
@@ -927,7 +928,7 @@ static int X11_ResizeWindow(_THIS,
 		} else {
 			if ( screen->flags & SDL_FULLSCREEN ) {
 				screen->flags &= ~SDL_FULLSCREEN;
-				X11_LeaveFullScreen(this);
+				X11_LeaveFullScreen(this, 0);
 			}
 		}
 		XResizeWindow(SDL_Display, SDL_Window, w, h);
@@ -1011,7 +1012,7 @@ static int X11_ToggleFullScreen(_THIS, int on)
 		X11_EnterFullScreen(this);
 	} else {
 		this->screen->flags &= ~SDL_FULLSCREEN;
-		X11_LeaveFullScreen(this);
+		X11_LeaveFullScreen(this, 0);
 	}
 	X11_RefreshDisplay(this);
 	if ( event_thread ) {
